@@ -32,6 +32,7 @@
 #include "mlview-xml-document.h"
 
 #include "mlview-utils.h"
+#include "mlview-xslt-utils.h"
 
 
 /**
@@ -179,9 +180,7 @@ static void view_name_changed_cb (MlViewIView * a_view,
 
 static GList *build_view_list_from_hashtable (GHashTable *a_views);
 
-static void free_mlview_xml_doc_views_hash_elements (gpointer a_key,
-                                                     gpointer a_value,
-                                                     gpointer a_user_data) ;
+static GList *build_doc_list_from_hashtable (GHashTable * a_docs);
 
 static void add_hash_key_to_list (gpointer a_key,
                                   gpointer a_value, GList ** a_list);
@@ -302,10 +301,19 @@ mlview_editor_dispose (GObject *a_this)
          *destroy each remaining view.
          **/
         if (PRIVATE (editor)->mlview_xml_doc_views) {
-                g_hash_table_foreach
-                        (PRIVATE (editor)->mlview_xml_doc_views,
-                         free_mlview_xml_doc_views_hash_elements,
-                         editor);
+                GList *list_item = NULL ;
+                GList *view_list = build_view_list_from_hashtable 
+                        (PRIVATE (editor)->mlview_xml_doc_views) ;
+
+                for (list_item = view_list ;
+                     list_item ;list_item = list_item->next) {
+                        if (list_item->data 
+                            && MLVIEW_IS_IVIEW (list_item->data)) {
+                                mlview_editor_remove_xml_document_view 
+                                        (editor, MLVIEW_IVIEW 
+                                         (list_item->data)) ;
+                        }
+                }
                 g_hash_table_destroy
                         (PRIVATE (editor)->mlview_xml_doc_views);
                 PRIVATE (editor)->mlview_xml_doc_views = NULL;
@@ -368,38 +376,6 @@ mlview_editor_finalize (GObject *a_this)
  *
  *=================================================*/
 
-/**
- *Called on each element of the "mlview_xml_doc_views" hash table
- *of the #MlViewEditor class.
- *Removes the view from the editor.
- *function expects the list element to be a MlViewXMLDocumentView
- *
- *@param a_key the key of the current key/value pair of the hash table.
- *@param a_value the value of the current key/value pair of the hash table.
- *@param a_user_data a custom user data. Not used yet.
- */
-static void
-free_mlview_xml_doc_views_hash_elements (gpointer a_key,
-                                         gpointer a_value,
-                                         gpointer a_user_data)
-{
-        MlViewIView *mlview_xml_doc_view = NULL ;
-
-        MlViewEditor *editor = NULL;
-
-        g_return_if_fail (a_key);
-
-        mlview_xml_doc_view =  a_key;
-
-        editor = MLVIEW_EDITOR (a_user_data);
-
-        g_return_if_fail (editor);
-
-        mlview_editor_remove_xml_document_view (editor,
-                                                mlview_xml_doc_view);
-}
-
-
 /**
  *Builds the contextual menu that appears when the user
  *right clicks on mlview.
@@ -778,6 +754,16 @@ build_view_list_from_hashtable (GHashTable * a_views)
         return result;
 }
 
+static GList *
+build_doc_list_from_hashtable (GHashTable * a_docs)
+{
+        GList *result = NULL;
+
+        g_hash_table_foreach (a_docs,
+                              (GHFunc) add_hash_key_to_list,
+                              &result);
+        return result;
+}
 
 static void
 add_hash_key_to_list (gpointer a_key,
@@ -796,7 +782,7 @@ add_hash_key_to_list (gpointer a_key,
  *the standard type builder of #MlViewEditor 
  *@return the type id of #MlViewEditor.
  */
-guint
+GType
 mlview_editor_get_type (void)
 {
         static guint type = 0;
@@ -940,8 +926,6 @@ mlview_editor_load_xml_file (MlViewEditor * a_this,
                           PRIVATE (a_this)->app_context)) ;
 
                 g_return_if_fail (new_view != NULL);
-                PRIVATE (a_this)->cur_view =
-                        new_view;
 
                 parent_window = gtk_widget_get_toplevel
                         (GTK_WIDGET (a_this));
@@ -955,9 +939,7 @@ mlview_editor_load_xml_file (MlViewEditor * a_this,
                                     (mlview_editor_event_cb),
                                     a_this);
                 mlview_editor_add_xml_document_view
-                        (a_this,
-                         MLVIEW_IVIEW
-                         (PRIVATE (a_this)->cur_view)) ;
+                        (a_this, new_view) ;
         }
 
         /*
@@ -999,12 +981,6 @@ mlview_editor_open_local_xml_document_interactive (MlViewEditor *a_this)
                 (PRIVATE (a_this)->app_context,
                  _("Choose the xml file to open"));
 
-        gtk_widget_realize (GTK_WIDGET (file_selector));
-        mlview_app_context_set_window_icon (PRIVATE (a_this)->
-                                            app_context,
-                                            GTK_WINDOW
-                                            (file_selector));
-
         button = mlview_file_selection_run
                 (MLVIEW_FILE_SELECTION (file_selector), TRUE);
 
@@ -1145,6 +1121,25 @@ mlview_editor_get_cur_doc (MlViewEditor * a_this)
         return doc ;
 }
 
+/**
+ *Getter of the glist of opened docs
+ *#MlViewXMLDocument *
+ *
+ *@param a_this the current instance of #MlViewEditor.
+ *@return a GList* of #MlViewXMLDocument, taht the caller 
+ * is responsble to free
+ */
+GList *
+mlview_editor_get_list_open_doc (MlViewEditor * a_this)
+{
+   GList *docs = NULL;
+
+   docs = build_doc_list_from_hashtable
+                (PRIVATE (a_this)->mlview_xml_docs);
+
+   return docs;
+}
+
 /**
  *Setter of the name of the currently selected view.
  *
@@ -1227,9 +1222,6 @@ mlview_editor_add_xml_document_view (MlViewEditor * a_this,
         g_return_if_fail (view_impl) ;
         g_return_if_fail (PRIVATE (a_this)->mlview_xml_docs);
         g_return_if_fail (PRIVATE (a_this)->mlview_xml_doc_views);
-        
-        if (PRIVATE (a_this)->cur_view == NULL)
-                PRIVATE (a_this)->cur_view = a_view;
 
         mlview_iview_get_document (a_view, &mlview_xml_document) ;
         g_return_if_fail (mlview_xml_document != NULL);
@@ -1238,13 +1230,13 @@ mlview_editor_add_xml_document_view (MlViewEditor * a_this,
                 (mlview_xml_document) ;
         if (file_desc)
                 file_path =
-                        mlview_file_descriptor_get_file_path 
-                        (file_desc);        
+                        mlview_file_descriptor_get_file_path
+                        (file_desc) ;
         /*check if a view on the same document has been added to
          *the editor already.
          */
         views_associated_to_document =
-                g_hash_table_lookup 
+                g_hash_table_lookup
                 (PRIVATE (a_this)->mlview_xml_docs,
                  mlview_xml_document);
 
@@ -1450,6 +1442,10 @@ mlview_editor_add_xml_document_view (MlViewEditor * a_this,
                           "name-changed",
                           G_CALLBACK (view_name_changed_cb),
                           a_this);
+        /*make sure the newly added view comes on "front"*/
+        gtk_widget_show (view_impl) ;
+        gtk_notebook_set_current_page
+                (PRIVATE (a_this)->notebook, -1) ;
         mlview_tree_view_set_all_paned_proportions
                 (MLVIEW_TREE_VIEW (a_view), 50, 70) ;
         gtk_widget_show_all (GTK_WIDGET (a_this));
@@ -1610,6 +1606,71 @@ mlview_editor_remove_xml_document_view (MlViewEditor * a_this,
         }
 }
 
+/**
+ *Displays a dialog with an entry for the root element name.
+ *It's up to the caller check the validity of the name.
+ *
+ *@param elname the location to return the name string.
+ *@return TRUE if the user pressed OK, FALSE otherwise
+ */
+static gboolean
+mlview_editor_ask_root_element_name (gchar **elname)
+{
+        GtkWidget *dialog;
+        GtkWidget *hbox;
+        GtkWidget *image;
+        GtkWidget *label;
+        GtkWidget *entry;
+        gint res;
+        gboolean ret = FALSE;
+
+        g_return_val_if_fail (elname, FALSE);
+
+        *elname = NULL;
+
+        dialog = gtk_dialog_new_with_buttons 
+                (_("Name of the root element"),
+                 NULL, GTK_DIALOG_MODAL,
+                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
+                 GTK_STOCK_OK, GTK_RESPONSE_OK,
+                 NULL);
+
+        gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+        gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+        hbox = gtk_hbox_new (FALSE, 6);
+        image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION,
+                                          GTK_ICON_SIZE_DIALOG);
+        gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+        label = gtk_label_new (_("Root element name:"));
+        entry = gtk_entry_new ();
+        gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+
+        gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+                            hbox, FALSE, FALSE, 0);
+
+        gtk_widget_show_all (dialog);
+
+        res = gtk_dialog_run (GTK_DIALOG (dialog));
+        switch (res) {
+        case GTK_RESPONSE_OK:
+                *elname = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+                ret = TRUE;
+                break;
+        case GTK_RESPONSE_CANCEL:
+        case GTK_RESPONSE_DELETE_EVENT:
+		break;
+        default:
+                g_assert_not_reached ();
+	}
+
+        gtk_widget_destroy (dialog);
+        return ret;
+}
+
 
 /**
  *Interactively create a new document.
@@ -1621,148 +1682,92 @@ mlview_editor_remove_xml_document_view (MlViewEditor * a_this,
 void
 mlview_editor_create_new_xml_document (MlViewEditor * a_this)
 {
+        struct MlViewAppSettings *settings = NULL;
         MlViewXMLDocument *mlview_doc = NULL;
-        GtkWidget *label = NULL,
-                *entry = NULL,
-                *table = NULL;
-        gint res = 0;
-        GtkWidget *dialog = NULL;
-        guchar *elname = NULL,
-                *utf8_elname = NULL;
         xmlDocPtr xml_doc = NULL;
         xmlNodePtr xml_node = NULL;
-        struct MlViewAppSettings *settings = NULL ;
+        MlViewIView *view = NULL ;
         gboolean loop = TRUE;
+        gchar *utf8_elname = NULL;
 
         g_return_if_fail (a_this != NULL);
         g_return_if_fail (MLVIEW_IS_EDITOR (a_this));
         g_return_if_fail (PRIVATE (a_this) != NULL);
-        g_return_if_fail (PRIVATE (a_this)->app_context !=
-                          NULL);
+        g_return_if_fail (PRIVATE (a_this)->app_context != NULL);
 
         settings = mlview_app_context_get_settings 
-                (PRIVATE (a_this)->app_context) ;
-        g_return_if_fail (settings) ;
-        dialog = gtk_dialog_new_with_buttons 
-                (_("Name of the root element"),
-                 NULL,GTK_DIALOG_MODAL, 
-                 GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
-                 GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-                 NULL) ;
-        g_return_if_fail (dialog) ;        
-        label = gtk_label_new (_("Root element name:"));
-        entry = gtk_entry_new ();
-        gtk_entry_set_activates_default (GTK_ENTRY (entry),
-                                        TRUE) ;
-        table = gtk_table_new (1, 2, TRUE);
+                (PRIVATE (a_this)->app_context);
+        g_return_if_fail (settings);
 
-        gtk_table_attach_defaults (GTK_TABLE (table), label,
-                                   0, 1, 0, 1);
-        gtk_table_attach_defaults (GTK_TABLE (table), entry,
-                                   1, 2, 0, 1);
-        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-                            table, TRUE, TRUE, 0);
-        gtk_widget_show_all (table);
-        mlview_app_context_set_window_icon 
-                (PRIVATE (a_this)->app_context,
-                 GTK_WINDOW (dialog));
-        gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-                                         GTK_RESPONSE_ACCEPT) ;
         while (loop) {
-                res = gtk_dialog_run (GTK_DIALOG (dialog));
-                switch (res) {
-                case GTK_RESPONSE_ACCEPT: /*pressed ok */
-                        elname = (guchar *)
-                                gtk_entry_get_text (GTK_ENTRY
-                                                    (entry));
+                gboolean res = FALSE;
+                enum MlViewStatus status = MLVIEW_OK;
+                guchar *name_end = NULL;
 
-                        if (elname && *elname) {
-                                enum MlViewStatus status =
-                                        MLVIEW_OK;
-                                status = mlview_utils_isolat1_str_to_utf8 (elname, &utf8_elname);
+                res = mlview_editor_ask_root_element_name 
+                        (&utf8_elname);
 
-                                if (status != MLVIEW_OK) {
-                                        mlview_utils_trace_debug
-                                                ("utf8 conversion failed");
-                                        goto release_resources;
-                                }
-                                loop = FALSE;
+                /*the user hit cancel */
+                if (!res)
+                        break;
+                /*empty root name: show the dialog again */
+                if (!utf8_elname)
+                        continue;
 
-                        } else {
-                                loop = TRUE;
-                                utf8_elname = NULL;
-                        }
+                if (mlview_utils_is_white_string (utf8_elname) == TRUE)
+                        continue;
 
-                        if (utf8_elname
-                            &&
-                            !mlview_utils_is_white_string
-                            (utf8_elname)) {
-                                xml_node =
-                                        xmlNewNode (NULL,
-                                                    utf8_elname);
-                                xml_doc = xmlNewDoc ("1.0");
-                                xml_doc->name =
-                                        g_strdup ("untitled");
-                                xmlDocSetRootElement (xml_doc,
-                                                      xml_node);
+                /*check if is a valid element name */
+                status = mlview_utils_parse_element_name (utf8_elname,
+                                                          &name_end);
+                if (status != MLVIEW_OK || !name_end) {
+                        mlview_app_context_error (PRIVATE (a_this)->app_context,
+                                _("The string entered is not a well formed element name !!"));
+                        continue;
+                }
 
-                                mlview_doc =
-                                        mlview_xml_document_new
-                                        (xml_doc,
-                                         PRIVATE (a_this)->
-                                         app_context);
+                xml_node = xmlNewNode (NULL, utf8_elname);
+                xml_doc = xmlNewDoc ("1.0");
+                xml_doc->name = g_strdup ("untitled");
+                xmlDocSetRootElement (xml_doc, xml_node);
 
+                mlview_doc = mlview_xml_document_new (xml_doc,
+                                                      PRIVATE (a_this)->
+                                                      app_context);
+                g_return_if_fail (mlview_doc != NULL);
 
-                                if (mlview_doc != NULL
-                                    && settings->validation_is_on 
-                                    == TRUE) {
-                                        mlview_xml_document_associate_dtd_interactive
-                                                (mlview_doc);
-                                }
-                                gtk_widget_destroy
-                                        (GTK_WIDGET (dialog)) ;
-
-                                if (settings->validation_is_on == TRUE
-                                    && xml_node->type 
-                                    == XML_ELEMENT_NODE) {
-                                        mlview_parsing_utils_build_required_attributes_list
+                /*validation */
+                if (settings->general.validation_is_on == TRUE) {
+                        mlview_xml_document_associate_dtd_interactive
+                                       (mlview_doc);
+                        if (xml_node->type == XML_ELEMENT_NODE) {
+                                mlview_parsing_utils_build_required_attributes_list
                                                 (PRIVATE (a_this)->app_context,
                                                  xml_node);
 
-                                        mlview_parsing_utils_build_required_children_tree
+                                mlview_parsing_utils_build_required_children_tree
                                                 (PRIVATE
                                                  (a_this)->app_context,
                                                  &xml_node);
-                                }
-                                if (mlview_doc != NULL) {
-                                        MlViewIView *view = NULL ;
-
-                                        view = MLVIEW_IVIEW
-                                                (mlview_tree_view_new 
-                                                 (mlview_doc, 
-                                                  (gchar *) "", 
-                                                  PRIVATE
-                                                  (a_this)->app_context));
-                                        mlview_editor_add_xml_document_view
-                                                (a_this, view);
-                                }
                         }
-                        break;
-
-                case GTK_RESPONSE_REJECT:
-                default:
-                        loop = FALSE;
-                        gtk_widget_destroy (GTK_WIDGET (dialog));
-                        break;
                 }
+
+                view = MLVIEW_IVIEW (mlview_tree_view_new (mlview_doc, 
+                                            (gchar *) "", 
+                                            PRIVATE (a_this)->app_context));
+                                            mlview_editor_add_xml_document_view
+                                                    (a_this, view);
+                /*everything went fine */
+                break;
         }
-      release_resources:
+
         if (utf8_elname) {
                 g_free (utf8_elname);
-                utf8_elname = NULL;
+                utf8_elname = NULL ;
         }
 }
 
+
 /**
  *Edits the xml document given in argument.
  *Actually, this method creates a view on this document and
@@ -1813,12 +1818,14 @@ mlview_editor_execute_action (MlViewEditor *a_this,
 
         g_return_val_if_fail (a_this
                               && MLVIEW_IS_EDITOR (a_this)
-                              && PRIVATE (a_this)
-                              && PRIVATE (a_this)->cur_view,
+                              && PRIVATE (a_this),
                               MLVIEW_BAD_PARAM_ERROR) ;
 
-        status = mlview_iview_execute_action
-                (PRIVATE (a_this)->cur_view, a_action) ;
+        if (PRIVATE (a_this)->cur_view) {
+                status = mlview_iview_execute_action
+                        (PRIVATE (a_this)->cur_view,
+                         a_action) ;
+        }
         return status ;
 }
 
@@ -1996,10 +2003,6 @@ mlview_editor_save_xml_document_as_interactive (MlViewEditor *a_this)
 
         g_return_if_fail (file_sel);
 
-        mlview_utils_set_window_deco (GTK_WINDOW (file_sel),
-                                      PRIVATE (a_this)->
-                                      app_context, "MlView");
-
         button = mlview_file_selection_run (file_sel, TRUE);
 
         switch (button) {
@@ -2076,9 +2079,7 @@ mlview_editor_save_and_close_xml_document (MlViewEditor *a_this)
 
         file_desc = mlview_xml_document_get_file_descriptor 
                 (mlview_xml_document) ;
-        tmp_str = mlview_file_descriptor_get_file_path (file_desc) ;
-        if (!file_desc 
-            || !tmp_str) {
+        if (!file_desc) {
                 /*
                  *No file is associated to the document.
                  *=>Ask the user which file she wants to save
@@ -2089,9 +2090,6 @@ mlview_editor_save_and_close_xml_document (MlViewEditor *a_this)
                          _("Choose a xml document"));
                 g_return_if_fail (file_sel);
 
-                mlview_app_context_set_window_icon 
-                        (PRIVATE (a_this)->app_context,
-                         GTK_WINDOW (file_sel));
                 button = mlview_file_selection_run (file_sel, TRUE);
 
                 switch (button) {
@@ -2107,7 +2105,9 @@ mlview_editor_save_and_close_xml_document (MlViewEditor *a_this)
                 }
         } else {
                 /*The document already has*/
-                file_name = g_strdup (tmp_str) ;
+                tmp_str = mlview_file_descriptor_get_file_path (file_desc);
+                g_return_if_fail (tmp_str);
+                file_name = g_strdup (tmp_str);
         }
         if (file_name && strcmp (file_name, "")) {
                 mlview_xml_document_save
@@ -2120,7 +2120,6 @@ mlview_editor_save_and_close_xml_document (MlViewEditor *a_this)
                 g_free (file_name);
                 file_name = NULL;
         }
-        
 }
 
 
@@ -2158,6 +2157,55 @@ mlview_editor_close_all_xml_documents_interactive (MlViewEditor *a_this)
 }
 
 
+/**
+ *Shows a confirmation dialog before closing a modified document.
+ *
+ *@param a_this the current mlview editor.
+ */
+static void
+mlview_editor_confirm_close (MlViewEditor *a_this)
+{
+        GtkWidget *dialog;
+        gint ret;
+
+        g_return_if_fail (a_this != NULL);
+        g_return_if_fail (MLVIEW_IS_EDITOR (a_this));
+        g_return_if_fail (PRIVATE (a_this));
+
+        dialog = gtk_message_dialog_new (NULL,
+                                         GTK_DIALOG_MODAL,
+                                         GTK_MESSAGE_QUESTION,
+                                         GTK_BUTTONS_NONE,
+                                         _("The document has been modifed.\n"
+					   "Should I save it before closing it?"));
+
+        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                                _("_Close without Saving"), GTK_RESPONSE_NO,
+                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                GTK_STOCK_SAVE, GTK_RESPONSE_YES, NULL);
+
+        gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+        gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
+
+        ret = gtk_dialog_run (GTK_DIALOG (dialog));
+        switch (ret) {
+        case GTK_RESPONSE_YES:
+                mlview_editor_save_and_close_xml_document (a_this);
+                break;
+        case GTK_RESPONSE_NO:
+                mlview_editor_close_xml_document_without_saving (a_this);
+                break;
+        case GTK_RESPONSE_CANCEL:
+        case GTK_RESPONSE_DELETE_EVENT:
+                break ;
+        default:
+                g_assert_not_reached ();
+        }
+
+        gtk_widget_destroy (dialog);
+}
+
+
 /**
  *Interactively closes the current document view.
  *Saves the underlying document if needed.
@@ -2168,56 +2216,26 @@ void
 mlview_editor_close_xml_document_interactive (MlViewEditor *a_this)
 {
         MlViewXMLDocument *doc = NULL;
-        GtkWidget *dialog = NULL, *label = NULL;
-        int res = 0;
 
         g_return_if_fail (a_this != NULL);
         g_return_if_fail (MLVIEW_IS_EDITOR (a_this));
         g_return_if_fail (PRIVATE (a_this));
 
+        if (!PRIVATE (a_this)->cur_view)
+                return ;
         mlview_iview_get_document (PRIVATE (a_this)->cur_view,
                                    &doc) ;
-        g_return_if_fail (doc != NULL);
-
-        if (mlview_xml_document_needs_saving (doc) == FALSE) {
-                mlview_editor_close_xml_document_without_saving
-                        (a_this);
-                return;
-        }
-
-        dialog = gtk_dialog_new_with_buttons 
-                (_("Document has changed"),
-                 NULL,
-                 GTK_DIALOG_MODAL,
-                 _("Close without saving"), GTK_RESPONSE_REJECT,
-                 _("Save before closing"), GTK_RESPONSE_ACCEPT,
-                 NULL);
-        g_return_if_fail (dialog) ;
-        gtk_dialog_set_default_response (GTK_DIALOG (dialog),
-                                         GTK_RESPONSE_ACCEPT) ;
-        gtk_window_set_wmclass (GTK_WINDOW (dialog),
-                                "document-has-changed-dialog",
-                                "MlView");
-        label = gtk_label_new
-                (_("The document has been modifed. Should I save it before closing it?"));
-        gtk_widget_show_all (label);
-        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-                            label, TRUE, TRUE, 0);
-        gtk_widget_realize (GTK_WIDGET (dialog));
-        mlview_app_context_set_window_icon
-                (PRIVATE (a_this)->app_context,
-                 GTK_WINDOW (dialog));
-
-        res = gtk_dialog_run (GTK_DIALOG (dialog));
-        gtk_widget_destroy (dialog) ;
-        if (res == GTK_RESPONSE_ACCEPT) {
-                mlview_editor_save_and_close_xml_document
-                        (a_this);
-        } else if (res == GTK_RESPONSE_REJECT) {
-                mlview_editor_close_xml_document_without_saving
-                        (a_this);
+        if (!doc) {
+                mlview_utils_trace_info
+                        ("The current view has no associated document. "
+                         "This is truly weird, something bad is happening.") ;
+                return ;
         }
 
+        if (mlview_xml_document_needs_saving (doc) == FALSE)
+                mlview_editor_close_xml_document_without_saving (a_this);
+        else
+                mlview_editor_confirm_close (a_this);
 }
 
 /**
@@ -2294,3 +2312,34 @@ mlview_editor_validate (MlViewEditor * a_this)
 
         return mlview_xml_document_validate (doc);
 }
+
+
+/**
+ *Applies a XSLT stylesheet to the current document
+ *displaying a dialog for xslt selection and opens
+ *the transformation resulting document in the editor
+ *@param a_this the current instance of #MlViewEditor.
+ *
+ */
+void
+mlview_editor_xslt_transform_document_interactive (MlViewEditor *a_this)
+{
+        MlViewXMLDocument *src_doc = NULL;
+        MlViewXMLDocument *xsl_doc = NULL;
+        MlViewXMLDocument *res_doc = NULL;
+
+        g_return_if_fail (a_this != NULL);
+        g_return_if_fail (MLVIEW_IS_EDITOR (a_this));
+        g_return_if_fail (PRIVATE (a_this) != NULL);
+        
+        src_doc = mlview_editor_get_cur_doc (a_this);
+        xsl_doc = mlview_xslt_utils_select_xsl_doc (a_this);
+        if (xsl_doc != NULL) {
+                res_doc = mlview_xslt_utils_transform_document
+                        (src_doc, xsl_doc);
+                mlview_xml_document_unref(xsl_doc);
+                if (res_doc != NULL) 
+                        mlview_editor_create_new_view_on_document (a_this, res_doc, 
+                                                                   MLVIEW_XML_DOC_TREE_VIEW);
+        }
+}
