@@ -21,19 +21,13 @@
  *If not, write to the Free Software Foundation, 
  *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *
- *Copyright 2001-2002 dodji SEKETELI, Gal CHAMOULAUD.
- *http://www.freespiders.org
+ *See COPYRIGHT file for copyright information.
  */
 
-#define GTK_ENABLE_BROKEN
 #include <stdio.h>
-#include <locale.h>
-#include <ctype.h>
+#include <string.h>
+#include <libgnomeui/libgnomeui.h>
 #include "mlview-utils.h"
-#include <gnome.h>
-#include "config.h"
-#include <libxml/valid.h>
 
 /**
  *@file
@@ -93,43 +87,11 @@ static gint gv_base_array[] = {
 static void
 mlview_utils_init_i18n (void)
 {
-        bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
-        bind_textdomain_codeset (PACKAGE, "UTF-8") ;
-        textdomain (PACKAGE);
+        bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR) ;
+        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8") ;
+        textdomain (GETTEXT_PACKAGE);
 }
 
-/**
- *Sets the decoration of the window so that
- *it has got a decoration homegenous with the
- *other MlView windows. Basically, it sets the
- *the MlView icon for example.
- *Note that the windows must be *NOT* realized before
- *this function is called.
- *
- *@param a_w the instance of GtkWindow to decorate.
- *@param a_ctxt the current instance of #MlViewAppContext
- *@param a_wm_class_name the class name of the window as it
- *should be seen by the window manager. (just give a custom string
- *that identifies the type of window e.g error-dialog or file-selection-dialog
- *
- */
-void
-mlview_utils_set_window_deco (GtkWindow * a_w,
-                              MlViewAppContext * a_ctxt,
-                              const guchar * a_wm_class_name)
-{
-        g_return_if_fail (a_w && GTK_IS_WINDOW (a_w));
-        g_return_if_fail (a_ctxt && a_wm_class_name);
-
-        if (GTK_WIDGET_REALIZED (GTK_WIDGET (a_w)) == TRUE)
-                return;
-
-        gtk_window_set_wmclass (a_w, a_wm_class_name, "MlView");
-
-        gtk_widget_realize (GTK_WIDGET (a_w));
-
-        mlview_app_context_set_window_icon (a_ctxt, a_w);
-}
 
 /**
  *This has been copied from libxml2.
@@ -523,6 +485,113 @@ mlview_utils_is_name_char (gint a_c)
         return FALSE ;
 }
 
+enum MlViewStatus
+mlview_utils_parse_pi (guchar *a_raw_pi,
+                       GString **a_pi_target,
+                       GString **a_pi_param)
+{
+        guchar *cur=a_raw_pi ;
+        guchar *pi_param_start=NULL, 
+                *pi_param_end=NULL,
+                *pi_target_end=NULL;
+        GString *pi_target = NULL ;
+        gint len=0 ;
+        guint32 unichar = 0 ;
+        enum MlViewStatus status = MLVIEW_OK ;
+
+        g_return_val_if_fail (a_raw_pi
+                              && a_pi_target
+                              && (*a_pi_target == NULL)
+                              && a_pi_param
+                              && (*a_pi_param == NULL),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        len = strlen (a_raw_pi) ;
+        if (len < 5)
+                return MLVIEW_PARSING_ERROR ;
+        if (*cur      == '<'
+            && cur[1] == '?') {
+                cur += 2 ;
+        } else {
+                return MLVIEW_PARSING_ERROR ;
+        }
+        status = mlview_utils_parse_element_name
+                (cur, &pi_target_end) ;
+        if (status != MLVIEW_OK
+            || !pi_target_end) {
+                status = MLVIEW_PARSING_ERROR ;
+                goto cleanup ;
+        }        
+        pi_target = g_string_new_len
+                (cur, pi_target_end - cur + 1);
+        cur = pi_target_end + 1;
+        if (len <= (cur - a_raw_pi)
+            || len -  (cur - a_raw_pi) < 2) {
+                status = MLVIEW_PARSING_ERROR ;
+                goto cleanup ;
+        }
+        if (mlview_utils_is_space (*cur) == TRUE) {
+                status = mlview_utils_skip_spaces
+                        (cur, &cur) ;
+                if (status != MLVIEW_OK)
+                        goto cleanup ;
+                pi_param_start = cur ;
+        }
+
+        if (pi_param_start)
+                goto eat_chars ;
+
+        if (*cur      == '?'
+            && cur[1] == '>') {
+                cur += 2 ;
+                goto end_of_pi ;
+        } else {
+                status = MLVIEW_PARSING_ERROR ;
+                goto cleanup ;
+        }
+
+ eat_chars:
+        cur = g_utf8_find_next_char (cur, NULL) ;
+        unichar = g_utf8_get_char (cur) ;
+        if (!unichar) {
+                status = MLVIEW_EOF_ERROR ;
+                goto cleanup ;
+        }
+        if (unichar == '?') {
+                cur = g_utf8_find_next_char (cur, NULL) ;
+                unichar = g_utf8_get_char (cur) ;
+                if (unichar == '>') {
+                        pi_param_end = cur - 2 ;
+                        goto end_of_pi ;
+                } else if (!unichar) {
+                        status = MLVIEW_EOF_ERROR ;
+                        goto cleanup ;
+                } else {                        
+                        goto eat_chars ;
+                }
+        } else {
+                goto eat_chars ;
+        }
+
+ end_of_pi:
+        *a_pi_target = pi_target ;
+        pi_target = NULL ;
+        if (pi_param_start && pi_param_end) {
+                *a_pi_param = g_string_new_len
+                        (pi_param_start,
+                         pi_param_end - pi_param_start + 1) ;
+                pi_param_start = pi_param_end = NULL ;
+        }
+        status = MLVIEW_OK ;
+
+ cleanup:
+        if (pi_target) {
+                g_string_free (pi_target, TRUE) ;
+                pi_target = NULL ;
+        }
+        return status ;
+}
+
 enum MlViewStatus
 mlview_utils_parse_element_name (guchar *a_raw_str,
                                  guchar **a_name_end)
@@ -553,7 +622,7 @@ mlview_utils_parse_element_name (guchar *a_raw_str,
         }
         prev = cur ;
         cur = g_utf8_find_next_char (prev, NULL) ;
-        while (cur) {
+        while (cur && *cur) {
                 cur_unichar = g_utf8_get_char (cur) ;
                 if (!cur_unichar) {
                         status = MLVIEW_EOF_ERROR ;
@@ -815,6 +884,53 @@ mlview_utils_name_value_pair_list_free (GList *a_nv_pair_list,
         g_list_free (a_nv_pair_list) ;
 }
 
+enum MlViewStatus
+mlview_utils_parse_comment (guchar *a_raw_str,
+                            GString **a_comment)
+{
+        gint len=0, i=0 ;
+        guchar *cur = a_raw_str ;
+        guchar *comment_start = NULL ;
+        guchar *comment_end = NULL ;
+        gboolean parsing_ok = FALSE ;
+
+        g_return_val_if_fail (a_raw_str && a_comment
+                              && (*a_comment == NULL),
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        len = strlen (a_raw_str) ;
+        if (len < 7)
+                return MLVIEW_PARSING_ERROR ;
+        if (cur[i]      ==  '<'
+            && cur[i+1] ==  '!'
+            && cur[i+2] ==  '-'
+            && cur[i+3] ==  '-') {                
+                i += 4 ;
+                comment_start = &cur[i] ;
+        } else {
+                return MLVIEW_PARSING_ERROR ;
+        }
+        while (len - i >= 3) {
+                if (cur[i]        == '-'
+                    && cur[i + 1] == '-'
+                    && cur[i + 2] == '>') {
+                        parsing_ok = TRUE ;
+                        comment_end = &cur[i-1] ;
+                        break ;
+                }
+                i++ ;
+        }
+        *a_comment = g_string_new_len
+                (comment_start,
+                 comment_end - comment_start + 1) ;
+        if (!*a_comment) {
+                mlview_utils_trace_info 
+                        ("!*a_comment failed") ;
+                return MLVIEW_ERROR ;
+        }
+        return MLVIEW_OK ;
+}
+
 enum MlViewStatus
 mlview_utils_parse_start_tag (guchar *a_raw_str,
                               GString **a_name,
@@ -1011,6 +1127,89 @@ mlview_utils_is_white_string (const guchar * a_str)
         return TRUE;
 }
 
+/**
+ *Walks a string and escapes the '<' characters to &lt;
+ *and '>' into &gt; ;
+ *@param a_instr the input string to escape.
+ *@param a_outstr the output escaped string. *a_outstr must be
+ *freed by the caller.
+ *@param a_outstrlen the length of *a_outstr.
+ *@param MLVIEW_OK upon successful completion, an error code otherwise.
+ */
+enum MlViewStatus
+mlview_utils_escape_predef_entities_in_str (guchar *a_instr,
+                                            guchar **a_outstr,
+                                            guint *a_outstrlen)
+{
+        gint instrlen = 0, nb_ab = 0, outstrlen = 0, nb_amp = 0 ;
+        guchar *char_ptr = NULL, *outstr = NULL, *outp ;
+
+        g_return_val_if_fail (a_instr
+                              && a_outstr
+                              && a_outstrlen, 
+                              MLVIEW_BAD_PARAM_ERROR) ;
+        
+        instrlen = strlen (a_instr) ;
+        /*
+         *first, walk through a_instr to count the number
+         *of angle brackets.
+         */
+        for (char_ptr = a_instr ; *char_ptr ; char_ptr++) {
+                if (*char_ptr == '<' || *char_ptr == '>') {
+                        nb_ab++ ;    
+                } else if (*char_ptr == '&'
+                           && ! (*(char_ptr + 1)     == 'a'
+                                 && *(char_ptr + 2 ) == 'm'
+                                 && *(char_ptr + 3 ) == 'p'
+                                 && *(char_ptr + 4 ) == ';')
+                        ) {
+                        nb_amp ++ ;
+                }
+        }
+        if (nb_ab || nb_amp) {
+                outstrlen = instrlen + nb_ab * 4 + nb_amp * 5 + 1;
+                outstr = g_try_malloc (outstrlen) ;
+                if (!outstr) {
+                        mlview_utils_trace_info
+                                ("malloc failed. system may be out of mem\n") ;
+                        return MLVIEW_OUT_OF_MEMORY_ERROR ;
+                }                                
+        }
+
+        if (outstr) {
+                /*
+                 *Walk a_strin to escape '<' and '>'.
+                 */
+                for (char_ptr = a_instr, outp = outstr ;
+                     char_ptr && *char_ptr;
+                     char_ptr ++) {
+                        if (*char_ptr == '<') {
+                                *outp++ = '&' ;
+                                *outp++ = 'l' ;
+                                *outp++ = 't' ;
+                                *outp++ = ';' ;
+                        } else if (*char_ptr == '>') {
+                                *outp++ = '&' ;
+                                *outp++ = 'g' ;
+                                *outp++ = 't' ;
+                                *outp++ = ';' ;
+                        } else if (*char_ptr == '&') {
+                                *outp++ = '&' ;
+                                *outp++ = 'a' ;
+                                *outp++ = 'm' ;
+                                *outp++ = 'p' ;
+                                *outp++ = ';' ;
+                        } else {
+                                *outp++ = *char_ptr ;
+                        }
+                }
+                *outp = 0 ;
+        }
+        *a_outstrlen = outstrlen ;
+        *a_outstr = outstr ;
+
+        return MLVIEW_OK ;
+}
 
 /**
  *Given an ISOLAT1 string, this function
@@ -2048,6 +2247,103 @@ mlview_utils_parse_full_name (xmlNode * a_node,
         }
 }
 
+/**
+ *Parses the doctype declaration as specified at
+ *http://www.w3.org/TR/REC-xml#NT-doctypedecl, but ignores
+ *the markup declaration parts ...
+ *This is only to allow the user to edit the name, public id
+ *and system id.
+ *@param a_instr the input stream to parse
+ *@param a_name_start out parameter. A pointer to the first byte
+ *of the doctype declaration name.
+ *@param a_name_end out parameter. A pointer to the last byte of the
+ *doctype declaration name.
+ *@param a_public_id_start out parameter. A pointer to the first byte of 
+ *the parsed public_id if any, or NULL.
+ *@param a_public_id_end out parameter. A pointer to the last byte of 
+ *the parsed public_id if any, or NULL.
+ *@param a_system_id_start out paramter. A pointer ot the first byte of
+ *the parsed system_id if any, or NULL.
+ *@param a_system_id_end out paramter. A pointer ot the last byte of
+ *the parsed system_id if any, or NULL.
+ *@return MLVIEW_OK upon successful completion, an error code otherwise.
+ */
+enum MlViewStatus
+mlview_utils_parse_doctype_decl (guchar *a_instr,
+                                 guchar **a_name_start,
+                                 guchar **a_name_end,
+                                 guchar **a_public_id_start,
+                                 guchar **a_public_id_end,
+                                 guchar **a_system_id_start,
+                                 guchar **a_system_id_end) 
+{
+        guint instr_len = 0 ;
+        guchar *iptr = a_instr ;
+        guchar *name_start=NULL, *name_end=NULL ;
+        guchar *public_id_start=NULL,*public_id_end=NULL ;
+        guchar *system_id_start=NULL, *system_id_end=NULL;
+        enum MlViewStatus status = MLVIEW_OK ;
+
+        g_return_val_if_fail (a_instr 
+                              && a_name_start 
+                              && a_name_end
+                              && a_public_id_start
+                              && a_public_id_end
+                              && a_system_id_start
+                              && a_system_id_end,
+                             MLVIEW_BAD_PARAM_ERROR) ;
+        instr_len = strlen (a_instr) ;
+
+        if (instr_len < 11)
+                return MLVIEW_PARSING_ERROR ;
+
+        if (*iptr          == '<'
+            && *(iptr + 1) == '!'
+            && *(iptr + 2) == 'D'
+            && *(iptr + 3) == 'O'
+            && *(iptr + 4) == 'C'
+            && *(iptr + 5) == 'T'
+            && *(iptr + 6) == 'Y'
+            && *(iptr + 7) == 'P'
+            && *(iptr + 8) == 'E'
+            && mlview_utils_is_space (*(iptr + 9)) == TRUE) {
+                iptr += 10 ;
+        } else {
+                return MLVIEW_PARSING_ERROR ;
+        }
+        
+        status = mlview_utils_parse_element_name (iptr, &name_end) ;
+        if (status == MLVIEW_OK) {
+                name_start = iptr ;
+                iptr = name_end + 1 ;
+        } else {
+                return MLVIEW_PARSING_ERROR ;
+        }
+        
+        if (mlview_utils_is_space (*iptr) == TRUE) {
+                iptr++ ;
+                status = mlview_utils_parse_dtd_id (iptr, &public_id_start,
+                                                    &public_id_end, 
+                                                    &system_id_start,
+                                                    &system_id_end) ;
+        }
+        /*
+         *FIXME: now, ignore everything until '>' or untill ']S?>'
+         *if applicable.
+         */
+        return MLVIEW_OK ;
+}
+
+
+enum MlViewStatus
+mlview_utils_parse_dtd_id (guchar *a_instr,
+                           guchar **a_public_id_start,
+                           guchar **a_public_id_end,
+                           guchar **a_system_id_start,
+                           guchar **a_system_id_end)
+{
+        return MLVIEW_OK ;
+}
 
 /**
  *Displays an error message in a error dialog graphically.
@@ -2066,65 +2362,36 @@ mlview_utils_display_error_dialog (MlViewAppContext * a_ctxt,
                                    ...)
 {
         va_list params;
-        GtkDialog *err_dialog = NULL;
+        GtkWidget *err_dialog = NULL;
         GnomeApp *app = NULL;
-        GtkText *text_widget = NULL;
         gchar *err_msg = NULL;
 
         g_return_if_fail (a_msg_format != NULL);
-
         va_start (params, a_msg_format);
-
         err_msg = g_strdup_vprintf (a_msg_format, params);
-
         va_end (params);
 
         g_return_if_fail (err_msg != NULL);
-
         app = mlview_app_context_get_element (a_ctxt,
                                               "GnomeApp");
-
         g_return_if_fail (app && GTK_IS_WINDOW (app));
+        err_dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+                                             GTK_MESSAGE_ERROR,
+                                             GTK_BUTTONS_CLOSE,
+                                             err_msg) ;
 
-        text_widget = GTK_TEXT (gtk_text_new (NULL, NULL));
-
-        gtk_text_set_editable (text_widget, FALSE);
-
-        gtk_text_set_line_wrap (text_widget, TRUE);
-
-        gtk_text_set_point (text_widget, 0);
-
-        gtk_text_insert (text_widget, NULL, NULL, NULL,
-                         err_msg, strlen (err_msg));
-
-
-        err_dialog =
-                GTK_DIALOG (gtk_dialog_new_with_buttons
-                            (_("Error"), GTK_WINDOW (app),
-                             GTK_DIALOG_MODAL, _("OK"),
-                             GTK_RESPONSE_ACCEPT, NULL));
-
-        mlview_utils_set_window_deco (GTK_WINDOW (err_dialog),
-                                      a_ctxt, "error-dialog");
-
+        g_return_if_fail (err_dialog) ;
+        gtk_dialog_set_default_response 
+                (GTK_DIALOG (err_dialog), GTK_RESPONSE_ACCEPT) ;
         g_return_if_fail (err_dialog);
-
-        gtk_box_pack_start_defaults (GTK_BOX (err_dialog->vbox),
-                                     GTK_WIDGET (text_widget));
-
         /*let the window be user resizable */
         gtk_window_set_policy (GTK_WINDOW (err_dialog),
                                FALSE, TRUE, FALSE);
-
-        gtk_widget_show_all (GTK_WIDGET (err_dialog));
-
-        gtk_dialog_run (err_dialog);
-
-        gtk_widget_destroy (GTK_WIDGET (err_dialog));
-
+        gtk_dialog_run (GTK_DIALOG (err_dialog));
+        gtk_widget_destroy (err_dialog) ;
         if (err_msg) {
-                g_free (err_msg);
-                err_msg = NULL;
+                g_free (err_msg) ;
+                err_msg = NULL ;
         }
 }
 
@@ -2198,8 +2465,12 @@ mlview_utils_gtk_row_ref_2_iter (GtkTreeModel * a_model,
 /**
  *This is a gtk+ add on function.
  *It expands a given row to a given depth.
- *@param a_model the instance of #GtkTreeModel to work on.
- *@param
+ *@param a_view the GtkTreeView to consider.
+ *@param a_path the path of the row to expand.
+ *@param a_depth the expansion depth. If <0, expands
+ *the row untill the leaves.
+ *@return MLVIEW_OK upon successfull completion, an error code
+ *otherwise.
  */
 enum MlViewStatus
 mlview_utils_gtk_tree_view_expand_row_to_depth (GtkTreeView *a_view,
@@ -2230,7 +2501,7 @@ mlview_utils_gtk_tree_view_expand_row_to_depth (GtkTreeView *a_view,
                              (model, &iter)) {
                         GtkTreePath *tmp_path=NULL ;
                         GtkTreeIter children_iter = {0} ;
-                        tmp_path = gtk_tree_model_get_path 
+                        tmp_path = gtk_tree_model_get_path
                                 (model, &iter) ;
                         if (!tmp_path)
                                 continue ;
@@ -2241,10 +2512,10 @@ mlview_utils_gtk_tree_view_expand_row_to_depth (GtkTreeView *a_view,
                         is_ok = gtk_tree_model_iter_children
                                 (model,  &children_iter, &iter) ;
                         if (is_ok == TRUE) {
-                                tmp_path = gtk_tree_model_get_path 
+                                tmp_path = gtk_tree_model_get_path
                                         (model, &children_iter) ;
                                 status = mlview_utils_gtk_tree_view_expand_row_to_depth
-                                        (a_view, tmp_path, a_depth-1);
+                                        (a_view, tmp_path, a_depth-1) ;
                                 if (status != MLVIEW_OK) {
                                         g_warning ("argh, status == MLVIEW_OK failed.") ;
                                 }
@@ -2256,23 +2527,52 @@ mlview_utils_gtk_tree_view_expand_row_to_depth (GtkTreeView *a_view,
         return status ;
 }
 
-void
-mlview_utils_glade_widget_is_unable (gchar * a_file)
+
+enum MlViewStatus
+mlview_utils_tree_path_string_to_iter (GtkTreeModel *a_model,
+                                       guchar *a_tree_path_str,
+                                       GtkTreeIter *a_iter)
 {
-        GtkWidget *dialog;
-        gchar *err_str;
+        GtkTreePath *tree_path = NULL ;
 
-        err_str = g_strdup_printf
-                (_("Unable to load Glade user interface file; "
-                   "%s.\nMake sure the file is accessible."), a_file);
+        g_return_val_if_fail (a_tree_path_str 
+                              && a_iter
+                              && a_model,
+                              MLVIEW_BAD_PARAM_ERROR) ;
+                
+        tree_path = gtk_tree_path_new_from_string (a_tree_path_str) ;
+        g_return_val_if_fail (tree_path, MLVIEW_ERROR) ;
+        gtk_tree_model_get_iter (a_model, a_iter, tree_path) ;
+        if (tree_path) {
+                gtk_tree_path_free (tree_path) ;
+                tree_path = NULL ;
+        }
+        return MLVIEW_OK ;
+}
 
-        dialog = gtk_message_dialog_new
-                (NULL,
-                 GTK_DIALOG_DESTROY_WITH_PARENT,
-                 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, err_str);
+enum MlViewStatus
+mlview_utils_gtk_tree_view_expand_row_to_depth2 (GtkTreeView *a_view,
+                                                 GtkTreeIter *a_iter,
+                                                 gint a_depth)
+{
+        GtkTreePath *tree_path = NULL ;
+        GtkTreeModel * model = NULL ;
+        enum MlViewStatus status = MLVIEW_OK ;
 
-        gtk_dialog_run (GTK_DIALOG (dialog));
-
-        gtk_widget_destroy (dialog);
-        g_free (err_str);
+        g_return_val_if_fail (a_view 
+                              && GTK_IS_TREE_VIEW (a_view)
+                              && a_iter,
+                              MLVIEW_BAD_PARAM_ERROR) ;
+                
+        model = gtk_tree_view_get_model (a_view) ;
+        g_return_val_if_fail (model, MLVIEW_ERROR) ;
+        tree_path = gtk_tree_model_get_path (model, a_iter) ;
+        g_return_val_if_fail (tree_path, MLVIEW_ERROR) ;        
+        status = mlview_utils_gtk_tree_view_expand_row_to_depth 
+                (a_view, tree_path, a_depth) ;
+        if (tree_path) {
+                gtk_tree_path_free (tree_path) ;
+                tree_path = NULL ;
+        }
+        return status ;
 }
